############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import socket, urlparse
import ZSI
from twisted.python import log
from twisted.internet import reactor
from twisted.web.server import Site
from twisted.web.resource import Resource
from twisted.internet.error import CannotListenError

from pyGridWare.properties.ResourceProperties import *
from pyGridWare.addressing.AddressingUtils import AddressingUtils
# Enable HTTP chunking
from pyGridWare.utility.http import ChunkedHTTPChannel
Site.protocol = ChunkedHTTPChannel

# Generated Code
from pyGridWare.generated.services.basenotification.WS_BaseNotificationService import \
    WS_BaseNotificationServiceWSRF as NCServiceInterface
import pyGridWare.generated.types.notificationconsumerservice


class NotificationConsumerService(NCServiceInterface):

#    def __init__(self, post='/wsrf/services/NotificationConsumerService', **kw):
#        NCServiceInterface.__init__(self, post)

    def wsa_Notify(self, ps, address, **kw):
        response = NCServiceInterface.wsa_Notify(self, ps, address)

        log.msg("Notification: ", self.request)
        pyobj = self.request._NotificationMessage[0]
        log.msg("  NotificationMessage: ", pyobj)
        log.msg("    Topic: ", pyobj._Topic)
        log.msg("    ProducerReference: ", pyobj._ProducerReference)
        log.msg("    Message: ", pyobj._Message)
        pyobj = pyobj._Message._ResourcePropertyValueChangeNotification
        log.msg("        NewValue: ", pyobj._NewValue)
        log.msg("          Value: ", pyobj._NewValue._any)
        if pyobj._OldValue is not None:
            log.msg("        OldValue: ", pyobj._OldValue)
            log.msg("          Value: ", pyobj._OldValue._any)
        return response


class NotificationConsumerTwisted(object):

    def __init__(self, path='/wsrf/services/NotificationConsumerService', 
        notificationClass=NotificationConsumerService):
        '''
        notificationClass -- Notification Consumer Service 
        '''
        self.context = Context()
        self.path = path
        self.consumer = None
        self.factory = None
        self.port = None

        assert issubclass(notificationClass, NCServiceInterface), \
            'notificationClass must be NotificationConsumer'
        root = Resource()
        resource = root
        token = '/'
        ipath = path.strip(token)
        for key in ipath.split(token)[:-1]:
            resource.putChild(key, Resource())
            resource = resource.getStaticEntity(key)

        service_key = ipath.split(token)[-1]
#        self.consumer = notificationClass(post=path)
        self.consumer = notificationClass()
        resource.putChild(service_key, self.consumer)
        self.factory = Site(root)

    def getNotificationContext(self):
        return self._ctx
    def setNotificationContext(self, context):
        self._ctx = context
    context = property(getNotificationContext, setNotificationContext, \
       None, 'NotificationConsumer Context')

    def start(self, ssl=False, keyfile=None, certfile=None, contextFactory=None):
        scheme = 'http'
        self.consumer.ssl = ssl
        try:
            if ssl is True:
                if contextFactory is None:
                    from twisted.internet import ssl as SSL
                    contextFactory = SSL.DefaultOpenSSLContextFactory(keyfile, certfile)
                self.port = reactor.listenSSL(0, self.factory, contextFactory)
                scheme = 'https'
            else:
                self.port = reactor.listenTCP(0, self.factory)
        except CannotListenError, ex:
            raise
        hostname = socket.getfqdn(self.port.getHost().host)
        netloc = '%s:%s' %(hostname,self.port.getHost().port)
        url = urlparse.urlunparse((scheme,netloc,self.path,'','',''))
#        self.consumer.setServiceURL(url)
        self.context.setAddress(address=url)


"""
  Notification Consumer portType does not contain ResourceProperties
  so must manually set up a context.
"""

class Key(int):
    '''Helper classes for specifying SOAP Data
    '''
    typecode = ZSI.TC.Iint(pname=("http://wsrf.globus.org/",'NotificationConsumerKey'))


class Context:
    '''This is not done..  Need to use a ResourceHome
    to keep track of NotificationsConsumers.
    '''
    def __init__(self, address=None):
        self.resourceID = 2000
        self.address = address

    def setAddress(self, address):
        self.address = address

    def getConsumerReference(self):
        ref = ConsumerReference()
        ref.setAddress(self.address)
        ref.setReferenceProperties(self.resourceID)
        return ref


class ConsumerReference:
    '''Helper classes for specifying SOAP Data
    '''
    def __init__(self):
        self._Address = None
        self._ReferenceProperties = ReferenceProperties()

    def setAddress(self, address):
        self._Address = address

    def setReferenceProperties(self, resourceID):
        self._ReferenceProperties.setProperty(Key(resourceID))


class ReferenceProperties:
    '''Helper classes for specifying SOAP Data
    '''
    def __init__(self):
        self._any = []

    def setProperty(self, property):
        self._any.append(property)


NotificationConsumer = NotificationConsumerTwisted


