###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import time
from twisted.internet import reactor
from pyGridWare.resource.ResourceSweeper import ResourceSweeper
from pyGridWare.faults.PropertiesFaults import ResourceUnknownFault

class SimpleResourceHome:
    resourceID = 1000

    def __init__(self): 
        self.sweeper = ResourceSweeper(self, 5.0)
        self.resources = {}

    def save(self):
        return

    def add(self, ctx, key=None):
        if key:
            ctx.resourceID = key

        key = ctx.resourceID
        if int(key) > SimpleResourceHome.resourceID:
            SimpleResourceHome.resourceID = int(key)
            self.create()
        self.resources[str(key)] = ctx

    def create(self):
        key = SimpleResourceHome.resourceID 
        SimpleResourceHome.resourceID += 1
        return key

    def delete(self, key):
        del self.resources[str(key)]

    def setTerminationTime(self, key, terminationTime):
        self.resources[str(key)].TerminationTime = terminationTime

    def get(self, key):
        if self.resources.has_key(str(key)) is False:
            raise ResourceUnknownFault(\
                    timestamp=time.gmtime(),
                    originator=None,
                    errorcode=None,
                    description=None,
                    faultcause=None
            )
        return self.resources[str(key)]

    def getSweeperDelay(self):
        return self.sweeper.getSweeperDelay()

    def setSweeperDelay(self, delay):
        self.sweeper.setSweeperDelay(delay)

    def getResources(self):
        return self.resources.values()


class PersistentResourceHome(SimpleResourceHome):

    def __init__(self):
        from ConfigParser import ConfigParser, NoSectionError
        #reactor.addSystemEventTrigger('before', 'shutdown', self.save)
        SimpleResourceHome.__init__(self)

        section = '%s.%s' %(self.__class__.__module__,self.__class__.__name__)
        cp = ConfigParser()
        cp.read('persistent_resource_config.txt')
        try:
            gen = cp.items(section)
            # python2.3 does not throw NoSectionError above
            ctx_mod,ctx_class =  eval(cp.get(section, 'context'))
            prop_mod, prop_class =  eval(cp.get(section, 'properties'))
        except NoSectionError,ex:
            pass
        else:
            for i in ('from %s import %s' %(ctx_mod,ctx_class), \
                'from %s import %s' %(prop_mod,prop_class)):
                exec(i)
            ctx_class = eval(ctx_class)
            prop_class = eval(prop_class)
            for k,v in gen:
                if k in ('context','properties','url'): continue
                ctx = ctx_class(prop_class)
                ctx.properties.setUp(v)
                ctx.resourceID = int(k)
                self.add(ctx)
          
        del cp

    def save(self):
        l = []
        for key,context in self.resources.items():
            l.append('\n%s = %s' %(key, context.properties))

        if len(l) == 0:
            return

        f = open('persistent_resource_config.txt', 'w+')
        f.write('\n# save resource properties')
        f.write('\n[%s.%s]' %(self.__class__.__module__,self.__class__.__name__))
        f.writelines(l)
        f.write('\nurl = %s' %context.url)
        f.write('\ncontext = ("%s","%s")' \
            %(context.__class__.__module__, context.__class__.__name__))
        f.write('\nproperties = ("%s","%s")' \
            %(context.__class__.__module__, context.properties.__class__.__name__))
        f.write('\n\n# finish\n')
        f.close()


